@@ -23,6 +23,7 @@ my %mp4Map = (
 my %dirMap = (
     MOV => \%movMap,
     MP4 => \%mp4Map,
+    HEIC => { },    # can't currently write XMP to HEIC files
 );
 
 #------------------------------------------------------------------------------
@@ -46,7 +47,7 @@ sub IsCurPath($$)
 sub WriteQuickTime($$$)
 {
     my ($et, $dirInfo, $tagTablePtr) = @_;
-    my ($foundMDAT, @hold, $track);
+    my ($foundMDAT, $lengthChanged, @hold, $track);
     my $outfile = $$dirInfo{OutFile} or return 0;
     my $raf = $$dirInfo{RAF};
     my $dataPt = $$dirInfo{DataPt};
@@ -113,7 +114,10 @@ sub WriteQuickTime($$$)
         }
 
         # set flag if we have passed the 'mdat' atom
-        $foundMDAT = 1 if $tag eq 'mdat';
+        if ($tag eq 'mdat') {
+            $et->Error("Can't add tags. Multiple 'mdat' blocks!") if $foundMDAT and $lengthChanged;
+            $foundMDAT = 1;
+        }
 
         # rewrite this atom
         my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
@@ -135,7 +139,7 @@ sub WriteQuickTime($$$)
                 undef $tagInfo;
             }
         }
-        if ($tagInfo) {
+        if ($tagInfo and (not defined $$tagInfo{Writable} or $$tagInfo{Writable})) {
             # read the atom data
             $raf->Read($buff, $size) == $size or $et->Error("Error reading $tag data"), last;
             my $subdir = $$tagInfo{SubDirectory};
@@ -165,6 +169,7 @@ sub WriteQuickTime($$$)
                     HasData  => $$subdir{HasData},  # necessary?
                     Multi    => $$subdir{Multi},    # necessary?
                     OutFile  => $outfile,
+                    InPlace  => 2, # (to write fixed-length XMP if possible)
                 );
                 # pass the header pointer if necessary (for EXIF IFD's
                 # where the Base offset is at the end of the header)
@@ -205,18 +210,21 @@ sub WriteQuickTime($$$)
                 my $len = length $newData;
                 $len > 0x7ffffff7 and $et->Error("$tag to large to write"), last;
                 if ($len == $size or $dataPt or $foundMDAT) {
-                    # write the updated directory now
-                    Write($outfile, Set32u($len+8), $tag, $newData) or $rtnVal = 0, last;
-                    next;
+                    # write the updated directory now (unless length is zero, or it is needed as padding)
+                    if ($len or not $foundMDAT) {
+                        Write($outfile, Set32u($len+8), $tag, $newData) or $rtnVal = 0, last;
+                        $lengthChanged = 1 if $len != $size;
+                        next;
+                    }
                 } else {
                     # bad things happen if 'mdat' atom is moved (eg. Adobe Bridge crashes --
                     # there must be some absolute offsets somewhere that point into mdat),
                     # so hold this atom and write it out later
                     if ($len) {
                         push @hold, Set32u($len+8), $tag, $newData;
-                        $et->VPrint(0,"  Moving '$tag' atom to after 'mdat'");
+                        $et->VPrint(0,"  Moving '${tag}' atom to after 'mdat'");
                     } else {
-                        $et->VPrint(0,"  Freeing '$tag' atom (and zeroing data)");
+                        $et->VPrint(0,"  Freeing '${tag}' atom (and zeroing data)");
                     }
                     # write a 'free' atom here to keep 'mdat' at the same offset
                     substr($hdr, 4, 4) = 'free';
@@ -268,6 +276,7 @@ sub WriteQuickTime($$$)
                 }
                 my $newHdr = Set32u(8+length($newData)+length($uuid)) . $tag . $uuid;
                 Write($outfile, $newHdr, $newData) or $rtnVal = 0;
+                $lengthChanged = 1;
             }
             delete $$addDirs{$subName}; # add only once (must delete _after_ call to WriteDirectory())
         }
@@ -310,7 +319,12 @@ sub WriteMOV($$)
         $buff !~ /^(....)+(qt  )/s)
     {
         # file is MP4 format if 'ftyp' exists without 'qt  ' as a compatible brand
-        $ftype = 'MP4';
+        if ($buff =~ /^(heic|mif1|msf1|heix|hevc|hevx)/) {
+            $ftype = 'HEIC';
+            $et->Error("Can't currently write HEIC/HEIF files");
+        } else {
+            $ftype = 'MP4';
+        }
     } else {
         $ftype = 'MOV';
     }
@@ -345,7 +359,7 @@ QuickTime-based file formats like MOV and MP4.
 
 =head1 AUTHOR
 
-Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
